var controllers = angular.module('premieroutlet.controllers', ['premieroutlet.services']);

var forwardValidToComparator = function(a, b) {
  var aValidTo = Date.parse(a['valid_to']);
  var bValidTo = Date.parse(b['valid_to']);
  return aValidTo - bValidTo;
};

var backwardValidToComparator = function(a, b) {
  var aValidTo = Date.parse(a['valid_to']);
  var bValidTo = Date.parse(b['valid_to']);
  return bValidTo - aValidTo;
};

var forwardShopNameComparator = function(a, b) {
  return a['shopname'].localeCompare(b['shopname']);
};

var backwardShopNameComparator = function(a, b) {
  return b['shopname'].localeCompare(a['shopname']);
};

var forwardCategoryComparator = function(a, b) {
  return a['category'].localeCompare(b['category']);
};

var backwardCategoryComparator = function(a, b) {
  return b['category'].localeCompare(a['category']);
};


controllers.controller('CouponsCtrl', function($scope, $rootScope, $log, $ionicHistory, DataLoaderService, ListBuilderService) {

  if (!$scope.coupons) {
    ListBuilderService.buildValidCouponsList().then(function(couponList) {
      $scope.coupons = couponList;
    });
  }

  $scope.$on('$ionicView.enter', function(event) {
    $ionicHistory.clearHistory();
    if (window.analytics) {
      window.analytics.trackView('Kuponok');
    }
  });

  $scope.$on('setCouponListEvent', function(event, data) {
    $scope.coupons = data;
  });

  $scope.refreshCoupons = function() {
    var measuredTimeStart = new Date().getTime();
    DataLoaderService.syncEverything(true).then(function() {
      if (window.analytics) {
        window.analytics.trackTiming('BackendSync', new Date().getTime() - measuredTimeStart, 'everything', 'Minden adat frissítése');
      }
      ListBuilderService.buildValidCouponsList().then(function(couponList) {
        $scope.coupons = couponList;
        $scope.$broadcast('scroll.refreshComplete');
      }).catch(function(error) {
        $log.error('buildValidCouponsList error: ' + JSON.stringify(error));
        $scope.$broadcast('scroll.refreshComplete');
      });
    }).catch(function(error) {
      $log.error('syncEverything error: ' + JSON.stringify(error));
      $scope.$broadcast('scroll.refreshComplete');
    });
  };

  $scope.orderByValidity = function(event) {
    $scope.shopOrderClass = '';
    if ('poc_ordering-asc' == $scope.validityOrderClass || event.currentTarget.classList.contains('poc_ordering-asc')) {
      $scope.coupons.sort(backwardValidToComparator);
      $scope.validityOrderClass = 'poc_ordering-desc';
    }
    else if ('poc-ordering-desc' == $scope.validityOrderClass || event.currentTarget.classList.contains('poc_ordering-desc')) {
      $scope.coupons.sort(forwardValidToComparator);
      $scope.validityOrderClass = 'poc_ordering-asc';
    }
    else {
      $scope.coupons.sort(forwardValidToComparator);
      $scope.validityOrderClass = 'poc_ordering-asc';
    }
  };

  $scope.orderByShop = function(event) {
    $scope.validityOrderClass = '';
    if ('poc_ordering-asc' == $scope.shopOrderClass || event.currentTarget.classList.contains('poc_ordering-asc')) {
      $scope.coupons.sort(backwardShopNameComparator);
      $scope.shopOrderClass = 'poc_ordering-desc';
    }
    else if ('poc-ordering-desc' == $scope.shopOrderClass || event.currentTarget.classList.contains('poc_ordering-desc')) {
      $scope.coupons.sort(forwardShopNameComparator);
      $scope.shopOrderClass = 'poc_ordering-asc';
    }
    else {
      $scope.coupons.sort(forwardShopNameComparator);
      $scope.shopOrderClass = 'poc_ordering-asc';
    }
  };

});

controllers.controller('CouponDetailCtrl', function($q, $scope, $rootScope, $stateParams, $ionicPopup, $state, $log, $ionicHistory, Today, UserConfig, CouponDetailService, DataLoaderService, ListBuilderService, UserdataRemoteDb) {
  $scope.btnRedeemText = 'BEVÁLTÁS';

  CouponDetailService.getCoupon($stateParams.couponId).then(function(coupon) {
    CouponDetailService.getRedeemedCouponAmount($stateParams.couponId)
      .then(function(redeemedAmount) {
        coupon.amount -= redeemedAmount;
      }).catch(function(error) {
        $log.error('getRedeemedCouponAmount error: ' + JSON.stringify(error));
      }).finally(function() {
        $scope.coupon = coupon;
      });
    CouponDetailService.getMoreCoupons(coupon).then(function(moreCoupons) {
      $scope.moreCoupons = moreCoupons;
    });
    if (window.analytics) {
      window.analytics.trackView('Kupon: ' + coupon.shopname + ' - ' + coupon.summary + ' (' + coupon._id + ')');
    }
  });

  $scope.redeemCoupon = function(event) {
    var user = UserConfig.getData();
    if (!user) {
      if (window.analytics) {
        window.analytics.trackEvent('user_action', 'redeem_coupon_noreg', $scope.coupon.summary + ' / ' + $scope.coupon.shopname + ' (', $scope.coupon._id + ')', 0);
      }
      $ionicPopup.confirm({
        title: 'Beváltás',
        template: 'Beváltani csak regisztráció után lehet.',
        cancelText: 'Bezár',
        okText: 'Regisztráció'
      }).then(function(res) {
        if (res) {
          $state.go('tab.favorites');
        }
      });
      return;
    }
    if (user.needs_confirm) {
      if (window.analytics) {
        window.analytics.trackEvent('user_action', 'redeem_coupon_reg_not_confirmed', $scope.coupon.summary + ' / ' + $scope.coupon.shopname + ' (', $scope.coupon._id + ')', 0);
      }
      $ionicPopup.confirm({
        title: 'Beváltás',
        template: 'Nem igazoltad még vissza a regisztrációdat. Ellenőrizd a postafiókodat!',
        cancelText: 'Bezár',
        okText: 'Regisztráció'
      }).then(function(res) {
        if (res) {
          $state.go('tab.favorites');
        }
      });
      return;
    }
    if ($scope.coupon.valid_from > Today) {
      $ionicPopup.alert({
        title: 'Beváltás',
        template: "Beváltani csak {{coupon.valid_from|formatDate:'from'}} lehet!",
        scope: $scope
      });
      return;
    }
    $scope.data = {};
    var redeemPopup = $ionicPopup.show({
      template: '<input type="password" ng-model="data.couponPassword" >',
      title: 'Beváltási jelszó',
      subTitle: 'Kérd meg az eladót, hogy üsse be a jelszót!',
      scope: $scope,
      buttons: [
        {
          text: '<b>Beváltás</b>',
          type: 'button-positive',
          onTap: function(e) {
            if (!$scope.data.couponPassword) {
              e.preventDefault();
            }
            else {
              return $scope.data.couponPassword;
            }
          }
        },
        {
          text: 'Mégsem'
        }
      ]
    });
    redeemPopup.then(function(password) {
      if (password == undefined) {
        return;
      }
      $log.debug("password entered: " + password);
      if (password == $scope.coupon.password) {
        CouponDetailService.redeemCoupon($scope.coupon._id).then(function() {
          if (window.analytics) {
            window.analytics.trackEvent('user_action', 'redeem_coupon', $scope.coupon.summary + ' / ' + $scope.coupon.shopname + ' (', $scope.coupon._id + ')', 0);
          }
          UserdataRemoteDb.sync()
            .catch(function(error) {
              $log.error('UserdataRemoteDb.sync error: ' + JSON.stringify(error));
            });
          ListBuilderService.buildValidCouponsList().then(function(couponList) {
            $rootScope.$broadcast('setCouponListEvent', couponList);
          });
            $scope.btnRedeemText = "BEVÁLTVA";
          $ionicPopup.alert({
            title: 'Beváltás',
            template: 'Gratulálok! Olcsóbban tudod megvásárolni a kiválasztott termékeket!'
          }).then(function() {
            $state.go('tab.coupons');
          });
        })
      }
      else {
        $ionicPopup.alert({
          title: 'Beváltás',
          template: 'Hibás jelszó!'
        });
      }
    });
  };


  $scope.toggleFavourite = function(event) {
    var user = UserConfig.getData();
    if (!user) {
      if (window.analytics) {
        window.analytics.trackEvent('user_action', ($scope.coupon.favourite) ? ('unlike_coupon_noreg') : ('like_coupon_noreg'), $scope.coupon.summary + ' / ' + $scope.coupon.shopname + ' (', $scope.coupon._id + ')', 0);
      }
      $ionicPopup.alert({
        title: 'Kedvenc kupon',
        template: 'Kedvencnek jelölni csak regisztráció után lehet.'
      });
      return;
    }
    CouponDetailService.likeCoupon($scope.coupon._id).then(function(fav) {
      if (window.analytics) {
        window.analytics.trackEvent('user_action', (fav) ? ('like_coupon') : ('unlike_coupon'), $scope.coupon.summary + ' / ' + $scope.coupon.shopname + ' (', $scope.coupon._id + ')', 0);
      }
      $scope.coupon.favourite = fav;
      UserdataRemoteDb.sync()
        .catch(function(error) {
          $log.error('Userdata remote db error: ' + JSON.stringify(error));
        });
    });

  };

  $scope.goBack = function() {
    $ionicHistory.goBack();
  };

});


controllers.controller('CouponSimilarCtrl', function($scope, $rootScope, $stateParams, $ionicPopup, $state, $log, CouponDetailService, DataLoaderService, ListBuilderService) {
  $scope.coupon_id = $stateParams.couponId;
  $scope.loading = true;
  CouponDetailService.getSimilarCoupons($stateParams.couponId).then(function(couponList) {
    $scope.similarCoupons = couponList;
    if (window.analytics) {
      window.analytics.trackView('Hasonló kuponok: ' + coupon.shopname + ' - ' + coupon.summary + ' (' + coupon._id + ')');
    }
  }).catch(function(error) {
    $log.error('getSimilarCoupons error: ' + JSON.stringify(error));
  }).finally(function() {
    $scope.loading = false;
  });
});



controllers.controller('OffersCtrl', function($scope, $rootScope, $log, $ionicHistory, DataLoaderService, ListBuilderService) {

  if (!$scope.offers) {
    ListBuilderService.buildValidOffersList().then(function(offerList) {
      $scope.offers = offerList;
    });
  }

  $scope.$on('$ionicView.enter', function(event) {
    $ionicHistory.clearHistory();
    if (window.analytics) {
      window.analytics.trackView('Ajánlatok');
    }
  });

  $scope.$on('setOfferListEvent', function(event, data) {
    $scope.offers = data;
  });


  // With the new view caching in Ionic, Controllers are only called
  // when they are recreated or on app start, instead of every page change.
  // To listen for when this page is active (for example, to refresh data),
  // listen for the $ionicView.enter event:
  //
  //$scope.$on('$ionicView.enter', function(e) {
  //});
  $scope.refreshOffers = function() {
    var measuredTimeStart = new Date().getTime();
    DataLoaderService.syncEverything(true).then(function() {
      if (window.analytics) {
        window.analytics.trackTiming('BackendSync', new Date().getTime() - measuredTimeStart, 'everything', 'Minden adat frissítése');
      }
      measuredTimeStart = new Date().getTime();
      ListBuilderService.buildValidOffersList().then(function(offerList) {
        $scope.offers = offerList;
        $scope.$broadcast('scroll.refreshComplete');
      }).catch(function(error) {
        $log.error('buildValidOffersList error: ' + JSON.stringify(error));
        $scope.$broadcast('scroll.refreshComplete');
      });
    }).catch(function(error) {
      $log.error('syncEverything error: ' + JSON.stringify(error));
      $scope.$broadcast('scroll.refreshComplete');
    });
  };

  $scope.orderByValidity = function(event) {
    $scope.shopOrderClass = '';
    if ('poc_ordering-asc' == $scope.validityOrderClass || event.currentTarget.classList.contains('poc_ordering-asc')) {
      $scope.offers.sort(backwardValidToComparator);
      $scope.validityOrderClass = 'poc_ordering-desc';
    }
    else if ('poc-ordering-desc' == $scope.validityOrderClass || event.currentTarget.classList.contains('poc_ordering-desc')) {
      $scope.offers.sort(forwardValidToComparator);
      $scope.validityOrderClass = 'poc_ordering-asc';
    }
    else {
      $scope.offers.sort(forwardValidToComparator);
      $scope.validityOrderClass = 'poc_ordering-asc';
    }
  };

  $scope.orderByShop = function(event) {
    $scope.validityOrderClass = '';
    if ('poc_ordering-asc' == $scope.shopOrderClass || event.currentTarget.classList.contains('poc_ordering-asc')) {
      $scope.offers.sort(backwardShopNameComparator);
      $scope.shopOrderClass = 'poc_ordering-desc';
    }
    else if ('poc-ordering-desc' == $scope.shopOrderClass || event.currentTarget.classList.contains('poc_ordering-desc')) {
      $scope.offers.sort(forwardShopNameComparator);
      $scope.shopOrderClass = 'poc_ordering-asc';
    }
    else {
      $scope.offers.sort(forwardShopNameComparator);
      $scope.shopOrderClass = 'poc_ordering-asc';
    }
  };

});

controllers.controller('OfferDetailCtrl', function($scope, $stateParams, $ionicHistory, OfferDetailService) {
  OfferDetailService.getOffer($stateParams.offerId).then(function(offer) {
    $scope.offer = offer;
    OfferDetailService.getMoreOffers(offer).then(function(moreOffers) {
      $scope.moreOffers = moreOffers;
    });
    if (window.analytics) {
      window.analytics.trackView('Ajánlat: ' + offer.summary + ' / ' + offer.shopname + ' (' + offer._id + ')');
    }
  });

  $scope.goBack = function() {
    $ionicHistory.goBack();
  };

});

controllers.controller('OfferSimilarCtrl', function($scope, $rootScope, $stateParams, OfferDetailService) {
  $scope.offer_id = $stateParams.offerId;
  $scope.loading = true;
  OfferDetailService.getSimilarOffers($stateParams.offerId).then(function(offerList) {
    $scope.similarOffers = offerList;
    if (window.analytics) {
      window.analytics.trackView('Hasonló ajánlatok: ' + offer.summary + ' / ' + offer.shopname + ' (' + offer._id + ')');
    }
  }).finally(function() {
    $scope.loading = false;
  });
});




controllers.controller('ShopsCtrl', function($scope, $ionicPopup, $log, $ionicHistory, DataLoaderService, ListBuilderService, ShopDetailService) {

  if (!$scope.shops) {
    ListBuilderService.buildAllShops().then(function(shopList) {
      $scope.shops = shopList;
    });
  }

  $scope.$on('$ionicView.enter', function(event) {
    $ionicHistory.clearHistory();
    if (window.analytics) {
      window.analytics.trackView('Üzletek');
    }
  });

  $scope.$on('setShopListEvent', function(event, data) {
    $scope.shops = data;
  });

  $scope.filteredByFavourites = false;
  $scope.filteredByCategory = null;

  $scope.refreshShops = function() {
    var measuredTimeStart = new Date().getTime();
    DataLoaderService.syncEverything(true).then(function() {
      if (window.analytics) {
        window.analytics.trackTiming('BackendSync', new Date().getTime() - measuredTimeStart, 'everything', 'Minden adat frissítése');
      }
      ListBuilderService.buildAllShops().then(function(shopList) {
        $scope.shops = shopList;
        $scope.$broadcast('scroll.refreshComplete');
      });
    }).catch(function(error) {
      $log.error('refreshShops syncEverything error: ' + JSON.stringify(error));
      $scope.$broadcast('scroll.refreshComplete');
    });
  };

  $scope.filterByFavourite = function(event) {
    $scope.filteredByCategory = null;
    ListBuilderService.buildShopListByFavourite(!$scope.filteredByFavourites).then(function(shopList) {
      $scope.shops = shopList;
      $scope.filteredByFavourites = !$scope.filteredByFavourites;
    }).catch(function(error) {
      $log.error('Shop favourite filter failed: ' + JSON.stringify(error));
      $scope.filteredByFavourites = false;
    });
  };

  $scope.filterByCategory = function(event) {
    $scope.filteredByFavourites = false;
    if ($scope.filteredByCategory == null) {
      ShopDetailService.getCategories().then(function(categoryList) {
        $scope.categories = categoryList;
        $scope.selectCategory = function(category) {
          $scope.filteredByCategory = category;
          catSelectPopup.close();
          ListBuilderService.buildShopListByCategory(category).then(function(shopList) {
            $scope.shops = shopList;
          }).catch(function(error) {
            $log.error('Shop category filter failed: ' + JSON.stringify(error));
          });
        };
        var catSelectPopup = $ionicPopup.show({
          template: '<ion-list><ion-item ng-repeat="cat in categories" ng-click="selectCategory(cat)">{{cat}}</ion-item></ion-list>',
          title: 'Válassz kategóriát',
          scope: $scope,
          buttons: [
            {
              text: 'Mégsem'
            }
          ]
        });
      }).catch(function(error) {
        $log.error('filterByCategory getCategories error: ' + JSON.stringify(error));
      });
    }
    else {
      $scope.filteredByCategory = null;
      ListBuilderService.buildAllShops().then(function(shopList) {
        $scope.shops = shopList;
      }).catch(function(error) {
        $log.error('Shop category filter failed: ' + JSON.stringify(error));
      });
    }
  };

  $scope.orderByCategory = function(event) {
    $scope.favouriteOrderClass = '';
    if ('poc_ordering-asc' == $scope.categoryOrderClass || event.currentTarget.classList.contains('poc_ordering-asc')) {
      $scope.shops.sort(backwardCategoryComparator);
      $scope.categoryOrderClass = 'poc_ordering-desc';
    }
    else if ('poc-ordering-desc' == $scope.categoryOrderClass || event.currentTarget.classList.contains('poc_ordering-desc')) {
      $scope.shops.sort(forwardCategoryComparator);
      $scope.categoryOrderClass = 'poc_ordering-asc';
    }
    else {
      $scope.shops.sort(forwardCategoryComparator);
      $scope.categoryOrderClass = 'poc_ordering-asc';
    }
  };

});


controllers.controller('ShopDetailCtrl', function($scope, $rootScope, $stateParams, $ionicPopup, $ionicHistory, UserConfig, ShopDetailService, DataLoaderService, ListBuilderService, UserdataRemoteDb) {
  $scope.isFavourite = false;

  ShopDetailService.getShop($stateParams.shopId).then(function(shop) {
    $scope.shop = shop;
    if (window.analytics) {
      window.analytics.trackView('Üzlet: ' + shop.name + ' (' + shop._id + ')');
    }
  });

  $scope.toggleFavourite = function(event) {
    var user = UserConfig.getData();
    if (!user) {
      if (window.analytics) {
        window.analytics.trackEvent('user_action', ($scope.shop.favourite) ? ('unlike_shop_noreg') : ('like_shop_noreg'), $scope.shop.name + ' (', $scope.shop._id + ')', 0);
      }
      $ionicPopup.alert({
        title: 'Kedvenc üzlet',
        template: 'Kedvencnek jelölni csak regisztráció után lehet.'
      });
      return;
    }

    ShopDetailService.likeShop($scope.shop._id).then(function(fav) {
      $scope.shop.favourite = fav;
      UserdataRemoteDb.sync()
        .catch(function(error) {
          $log.error('toggleFavourite likeShop error: ' + JSON.stringify(error));
        })
    });
  };

  $scope.goBack = function() {
    $ionicHistory.goBack();
  };

});

controllers.controller('ShopSimilarCtrl', function($scope, $rootScope, $stateParams, ShopDetailService) {
  $scope.shop_id = $stateParams.shopId;
  $scope.loading = true;
  ShopDetailService.getSimilarShops($stateParams.shopId).then(function(shopList) {
    $scope.similarShops = shopList;
    if (window.analytics) {
      window.analytics.trackView('Hasonló üzletek: ' + shop.name + ' (' + shop._id + ')');
    }
  }).finally(function() {
    $scope.loading = false;
  });
});





controllers.controller('FavoritesCtrl', function($scope, $rootScope, $stateParams, $state, $ionicPopup, $ionicLoading, $log, $ionicHistory, UserConfig, UserdataDb, GeneralDb, FavouritesService, DataLoaderService, ListBuilderService, NotificationService) {
  $scope.$on('$ionicView.enter', function(event) {
    $ionicHistory.clearHistory();
    $scope.user = UserConfig.getData();
    if (window.analytics) {
      window.analytics.trackView('Kedvencek');
    }
  });


  $scope.fbLogin = function () {
    $scope.$emit('fbLogin');
  };

  $scope.gpLogin = function() {
    $scope.$emit('gpLogin');
  };

  /*
  $scope.fbLogout = function() {
    ngFB.logout().then(function() {
      $rootScope.user = undefined;
      $state.go($state.current, {}, {reload: true});
    });
  };
  */

  $scope.checkUserStatus = function() {
    var errorCatcher = function(error) {
      $log.error('checkUserStatus error: ' + JSON.stringify(error));
      if (window.analytics) {
        window.analytics.trackException('Own login error: ' + JSON.stringify(error), false);
      }
      $ionicLoading.hide();
    };

    $ionicLoading.show({
      template: 'Adatok letöltése...<br/><br/><ion-spinner icon="spiral" />'
    });

    FavouritesService.checkUserStatus().then(function(active) {
      if (active) {
        $scope.user = UserConfig.getData();
        if ($scope.user) {
          if (window.analytics) {
            window.analytics.setUserId($scope.user._id);
          }
          NotificationService.init(function (msg) {
            $ionicPopup.alert({
              title: 'Újdonságok',
              template: msg,
            }).then(function (res) {
              $state.go('tab.reminders');
            });
          });
        }

      }
      $ionicLoading.hide();
    }).catch(errorCatcher);
  };

  $scope.resendActivationEmail = function() {
    var errorCatcher = function(error) {
      $log.error('resendActivationEmail error: ' + JSON.stringify(error));
      if (window.analytics) {
        window.analytics.trackException('Activation email resend error: ' + JSON.stringify(error), false);
      }
      $ionicLoading.hide();
      $ionicPopup.alert({
        title: 'Emailküldés hiba',
        template: error + '<br/><br/>Próbálkozz később újra és ha továbbra sem sikerül, frissítsd az alkalmazást!'
      });
    };

    $ionicLoading.show({
      template: 'Email küldése...<br/><br/><ion-spinner icon="spiral" />'
    });

    FavouritesService.resendActivationEmail().then(function(success) {
      $ionicLoading.hide();
      if (!success.success) {
        $ionicPopup.alert({
          title: 'Emailküldés sikertelen',
          template: 'Nem sikerült elküldeni az email-t. Próbálkozz később újra!'
        });
      }
    }).catch(errorCatcher);
  };

  /*
  $scope.removeUserData = function() {
    FavouritesService.removeUserData().then(function() {
      $rootScope.user = false;
      $rootScope.favouriteCoupons = false;
      $rootScope.favouriteShops = false;
      $rootScope.favouriteProducts = false;
      $rootScope.messageList = false;
    }).catch(function(error) {
      $log.error(error);
    });
  };
  */

  $scope.delDbs = function() {
    UserdataDb.get().destroy();
    GeneralDb.get().destroy();
  };


});


controllers.controller('FavoriteShopsCtrl', function($scope, UserConfig, ListBuilderService) {
  $scope.$on('$ionicView.enter', function(event) {
    $scope.user = UserConfig.getData();
    ListBuilderService.buildFavouriteShopsList().then(function(shopList) {
      $scope.favouriteShops = shopList;
    });
    if (window.analytics) {
      window.analytics.trackView('Kedvenc üzletek');
    }
  });
});

controllers.controller('FavoriteProductsCtrl', function($log, $scope, $rootScope, $ionicPopup, UserConfig, FavouritesService, DataLoaderService, ListBuilderService) {
  $scope.$on('$ionicView.enter', function(event) {
    $scope.user = UserConfig.getData();
    ListBuilderService.buildFavouriteProductsList().then(function(productList) {
      $scope.favouriteProducts = productList;
    });
    if (window.analytics) {
      window.analytics.trackView('Kedvenc termékek');
    }
  });

  $scope.toggleFavourite = function(product) {
    FavouritesService.getProduct(product._id)
      .then(function(product) {
        var user = UserConfig.getData();
        if (!user) {
          if (window.analytics) {
            window.analytics.trackEvent('user_action', (product.favourite) ? ('unlike_product_noreg') : ('like_product_noreg'), product.name, 0);
          }
          $ionicPopup.alert({
            title: 'Kedvenc termék',
            template: 'Kedvencnek jelölni csak regisztráció után lehet.'
          });
          return;
        }

        FavouritesService.likeProduct(product._id)
          .then(function(fav) {
            ListBuilderService.buildFavouriteProductsList().then(function(productList) {
              $scope.favouriteProducts = productList;
            });
          })
          .catch(function(error) {
            $log.error('toggleFavourite likeProduct error: ' + JSON.stringify(error));
          });

      })
      .catch(function(error) {
        $log.error('toggleFavourite getProduct error: ' + JSON.stringify(error));
      });
  };

});

controllers.controller('FavoriteCouponsCtrl', function($scope, UserConfig, ListBuilderService) {
  $scope.$on('$ionicView.enter', function(event) {
    $scope.user = UserConfig.getData();
    ListBuilderService.buildFavouriteCouponsList().then(function(couponList) {
      $scope.favouriteCoupons = couponList;
    });
    if (window.analytics) {
      window.analytics.trackView('Kedvenc kuponok');
    }
  });
});

controllers.controller('RemindersCtrl', function($scope, $log, UserConfig, ListBuilderService) {
  $scope.$on('$ionicView.enter', function(event) {
    $scope.user = UserConfig.getData();
    ListBuilderService.buildMessagesList().then(function(messageList) {
      $log.debug('Loaded ' + messageList.length + ' messages');
      $scope.messageList = messageList;
    }).catch(function(error) {
      $log.error('RemindersCtrl buildMessagesList error: ' + JSON.stringify(error));
    })
    if (window.analytics) {
      window.analytics.trackView('Emlékeztetők');
    }
  });
});

controllers.controller('ReminderDetailCtrl', function($scope, $rootScope, $stateParams, $log, ReminderDetailService, UserConfig) {

  ReminderDetailService.getMessage($stateParams.messageId).then(function(message) {
    $scope.message = message;
    ReminderDetailService.markMessageAsRead($stateParams.messageId).then(function(messageList) {
      $rootScope.messageList = messageList;
    }).catch(function(error) {
      $log.error('ReminderDetailCtrl markMessageAsRead error: ' + JSON.stringify(error));
    });
    if (window.analytics) {
      window.analytics.trackView('Emlékeztető: ' + message.subject + ' (' + message._id + ')');
    }
  });

});

controllers.controller('LoginCtrl', function($scope, $rootScope, $state, $ionicLoading, $ionicPopup, $log, UserConfig, FavouritesService, ListBuilderService, NotificationService) {
  $scope.$on('$ionicView.enter', function(event) {
    if (window.analytics) {
      window.analytics.trackView('Login');
    }
  });
  $scope.userdata = {
    name: '',
    email: '',
    city: '',
    gender: '',
    birthyear: ''
  };

  $scope.ownLogin = function() {
    $scope.$emit('ownLogin');
  };

});
