var persistency = angular.module('premieroutlet.persistency', []);

persistency.factory('GeneralDb', function() {
  var db = null;
  var functions = {
    init: function() {
      PouchDB.enableAllDbs = true;
      db = new PouchDB('general', {auto_compaction: true});
    },
    get: function() {
      if (db == null) {
        functions.init();
      }
      return db;
    },
    destroy: function() {
      db.destroy();
      db = null;
    }
  };
  return functions;

});

persistency.factory('UserdataDb', function() {
  var db = null;
  var functions = {
    init: function() {
      PouchDB.enableAllDbs = true;
      db = new PouchDB('userdata', {auto_compaction: true});
    },
    get: function() {
      if (db == null) {
        functions.init();
      }
      return db;
    },
    destroy: function() {
      db.destroy();
      db = null;
    }
  };
  return functions;
});


persistency.factory('Config', function(VERSION, DESKTOPTEST, $q, $http, $log, GeneralDb, UserdataDb) {

  var config = null;

  var functions = {
    get: function() {
      return config;
    },

    load: function() {
      $log.debug('Config load started');
      var deferred = $q.defer();

      var setProxyUrls = function(config) {
        if (DESKTOPTEST) {
          var poRE = new RegExp('^https?://(www.)?premieroutlet.hu/');
          for (var property in config) {
            if (config.hasOwnProperty(property) && poRE.test(config[property])) {
              config[property] = config[property].substring(config[property].indexOf('/', 8));
            }
          }
        }
        return config;
      };

      var conf = window.localStorage.getItem('RemoteConfig');
      //var timestamp = window.localStorage.getItem('RemoteConfigTimestamp');
      var ver = window.localStorage.getItem('Version');
      //if (config && (new Date().getTime() - timestamp) < 1000 * 60 * 60 * 24 * 7) {
      if (conf && ver == VERSION) {
        $log.debug('Configuration loaded from local storage');
        config = JSON.parse(conf);
        config = setProxyUrls(config);
        deferred.resolve(config);
      }
      else {
        // if there is a version change, remove all local data
        if (conf && ver != VERSION) {
          $log.info('There is a version mismatch, existing databases will be removed.');
          GeneralDb.destroy();
          UserdataDb.destroy();
          window.localStorage.removeItem('RemoteConfig');
          window.localStorage.removeItem('RemoteConfigTimestamp');
          window.localStorage.removeItem('Version');
        }

        var url = 'http://www.premieroutlet.hu/mobile/config/' + VERSION;
        if (DESKTOPTEST) {
          url = url.substring(27);
        }
        $log.info('Loading config from ' + url);
        $http.get(url).then(function(response) {
          //$http.get('/mobile/config/' + VERSION).then(function(response) {
          response.data = setProxyUrls(response.data);
          window.localStorage.setItem('RemoteConfig', JSON.stringify(response.data));
          window.localStorage.setItem('RemoteConfigTimestamp', new Date().getTime());
          window.localStorage.setItem('Version', VERSION);
          config = response.data;
          $log.info('Config loaded: ' + JSON.stringify(config));
          deferred.resolve(config);
        }, function(error) {
          $log.error('Config file retrieval failed: ' + JSON.stringify(error));
          config = null;
          deferred.reject(error);
        });

      }
      return deferred.promise;
    }
  };
  return functions;

});


persistency.factory('UserConfig', function() {

  /** Reads object or array from local storage */
  var getObject = function(id) {
    return JSON.parse(window.localStorage.getItem(id));
  };
  var saveObject = function(id, obj) {
    window.localStorage.setItem(id, JSON.stringify(obj));
  };

  return {
    getId: function() {
      return window.localStorage.getItem('UserId');
    },
    saveId: function(id) {
      window.localStorage.setItem('UserId', id);
    },
    getData: function() {
      return getObject('UserData');
    },
    saveData: function(data) {
      saveObject('UserData', data);
    },
    saveStringData: function(data) {
      window.localStorage.setItem('UserData', data);
    },
    getNotificationRegistrationId: function() {
      return window.localStorage.getItem('NotificationRegistrationId');
    },
    saveNotificationRegistrationId: function(registrationId) {
      window.localStorage.setItem('NotificationRegistrationId', registrationId);
    },
    getReadMessageIds: function() {
      return getObject('ReadMessageIds');
    },
    isReadMessage: function(id) {
      var msgIds = getObject('ReadMessageIds');
      if (msgIds == null) {
        return false;
      }
      return msgIds.includes(id);
    },
    addReadMessageId: function(id) {
      var msgIds = getObject('ReadMessageIds');
      if (msgIds == null) {
        msgIds = [];
      }
      if (!msgIds.includes(id)) {
        msgIds.push(id);
      }
      saveObject('ReadMessageIds', msgIds);
    },
    removeReadMessageId: function(id) {
      var msgIds = getObject('ReadMessageIds');
      if (msgIds == null) {
        return;
      }
      var idx = msgIds.indexOf(id);
      if (idx > -1) {
        msgIds.splice(idx, 1);
      }
      saveObject('ReadMessageIds', msgIds);
    },
    cleanUp: function() {
      window.localStorage.removeItem('UserId');
      window.localStorage.removeItem('UserData');
    },
    getLastSyncTS: function() {
      var ts = Number(window.localStorage.getItem('LastSyncTS'));
      if (ts === null) {
        ts = 0;
      }
      return ts;
    },
    saveLastSyncTS: function() {
      window.localStorage.setItem('LastSyncTS', new Date().getTime());
    },
    getLastSeq: function() {
      return window.localStorage.getItem('LastSeq');
    },
    saveLastSeq: function(lastSeq) {
      window.localStorage.setItem('LastSeq', lastSeq);
    },
    removeLastSeq: function() {
      window.localStorage.removeItem('LastSeq');
    }
  };
});

persistency.factory('GeneralRemoteDb', function($q, $log, Today, Config, UserConfig, GeneralDb) {
  var db = null;
  var functions = {
    init: function() {
      PouchDB.enableAllDbs = true;
      db = new PouchDB(Config.get().generalDbUrl);
      $log.debug('GeneralRemoteDb initialized');
    },
    get: function() {
      if (db == null) {
        functions.init();
      }
      return db;
    },
    saveLastSequence: function() {
      db.changes({
        limit: 1,
        since: 'now'
      }).then(function(changes_result) {
        $log.debug('Changes results: ' + JSON.stringify(changes_result));
        UserConfig.saveLastSeq(changes_result.last_seq);
      }).catch(function(error) {
        $log.error('saveLastSequence error : ' + JSON.stringify(error));
      });
    },
    remove: function(id) {
      var deferred = $q.defer();
      var localdb = GeneralDb.get();
      localdb.get(id, {conflicts: true}).then(function(docToRemove) {
        $log.debug('Coupon to remove: ' + JSON.stringify(docToRemove));
        while (docToRemove._conflicts && docToRemove._conflicts.length > 0) {
          var conflicting_rev = docToRemove._conflicts.pop();
          localdb.remove(docToRemove._id, conflicting_rev).then(function(result) {
            $log.debug('Conflicting rev ' + conflicting_rev + ' deleted');
          }).catch(function(error) {
            deferred.reject(error);
          });
        }
        localdb.remove(docToRemove._id, docToRemove._rev).then(function(result) {
          deferred.resolve();
        }).catch(function(error) {
          deferred.reject(error);
        });
      }).catch(function(error) {
        ('Document could not be removed: ' + JSON.stringify(error));
        deferred.reject(error);
      });
      return deferred.promise;
    },
    sync: function() {
      var deferred = $q.defer();

      if (db == null) {
        functions.init();
      }
      var localdb = GeneralDb.get();
      var lastSeq = UserConfig.getLastSeq();
      var filter_options = {};
      if (lastSeq === null) {
        filter_options = {
          filter: 'app/byValidity',
          query_params: {
            timestamp: (new Date().getTime() / 1000).toFixed(0),
            isodate: Today
          }
        };
      }
      else {
        filter_options = {
          since: lastSeq
        };
      }
      $log.debug('Filter options: ' + JSON.stringify(filter_options));

      var changeCount = 0;
      localdb.replicate.from(db, filter_options)
        .on('change', function(info) {
          changeCount++;
          $log.debug('Replicated from remote general db: ' + info.docs.length + ' docs');
        })
        .then(function(replication_results) {
          $log.debug('Replication results: ' + JSON.stringify(replication_results));
          var replicated_ids = [];
          for (var i in replication_results.docs) {
            replicated_ids.push(replication_results.docs[i]._id);
          }
          localdb.allDocs({
            include_docs: true,
            keys: replicated_ids
          }).then(function(docs) {
            $log.debug('Replicated docs: ' + JSON.stringify(docs));
          }).catch(function(error) {
            ('Replicated docs cannot be retrieved: ' + JSON.stringify(error));
          });
          functions.saveLastSequence();
          UserConfig.saveLastSyncTS();
          console.log('General data replicated');
          deferred.resolve(changeCount);
        })
        .catch(function(err) {
          deferred.reject(err);
        });

      return deferred.promise;
    },
  };
  return functions;
});

persistency.factory('UserdataRemoteDb', function($q, $log, Config, UserConfig, UserdataDb) {
  var db = null;
  var functions = {
    init: function() {
      PouchDB.enableAllDbs = true;
      db = new PouchDB(Config.get().userdataDbUrl);
      $log.debug('UserdataRemoteDb initialized');
    },
    get: function() {
      if (db == null) {
        functions.init();
      }
      return db;
    },
    sync: function() {
      var deferred = $q.defer();

      var userId = UserConfig.getId();
      if (userId) {
        var localuserdatadb = UserdataDb.get();
        var syncoptions = {
          filter: 'userdata/allById',
          query_params: {
            user_id: userId
          }
        };
        localuserdatadb.get('_design/userdata')
          .then(function(doc) {
            localuserdatadb.sync(db, syncoptions)
              .then(function() {
                deferred.resolve();
              })
              .catch(function(error) {
                deferred.reject(error);
              });
          })
          .catch(function(error) {
            localuserdatadb.replicate.from(db, syncoptions)
              .then(function() {
                deferred.resolve();
              })
              .catch(function(error) {
                deferred.reject(error);
              });
          });

      }
      else {
        deferred.resolve();
      }

      return deferred.promise;
    },
  };
  return functions;
});

persistency.factory('BasicDAO', function($q, $log, GeneralDb) {
  var functions = {

    getRowsArrayPromise: function(promise) {
      var deferred = $q.defer();
      promise.then(function(result) {
        deferred.resolve(result.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    getDocById: function(id) {
      return GeneralDb.get().get(id, {attachments: true});
    },
    getDocsById: function(ids) {
      return functions.getRowsArrayPromise(GeneralDb.get().allDocs({
        keys: ids,
        include_docs: true,
        attachments: true
      }));
    }
  };
  return functions;
});

persistency.factory('CouponDAO', function($q, $log, UserConfig, GeneralDb, UserdataDb) {
  var functions = {
    getRowsArrayPromise: function(promise) {
      var deferred = $q.defer();
      promise.then(function(result) {
        deferred.resolve(result.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    getCouponsByValidity: function(validityStart) {
      return functions.getRowsArrayPromise(GeneralDb.get().query('coupons/byValidity', {
        startkey: validityStart,
        reduce: false,
        include_docs: true,
        attachments: true
      }));
    },
    getCouponsByCategory: function(category) {
      return functions.getRowsArrayPromise(GeneralDb.get().query('coupons/byCategory', {
        reduce: false,
        keys: category,
        include_docs: true,
        attachments: true
      }));
    },
    getCouponsByShop: function(shopId, validityStart) {
      var deferred = $q.defer();
      GeneralDb.get().query('coupons/byShop', {
        reduce: false,
        key: shopId,
        include_docs: true
      }).then(function(couponList) {
        for (var i = 0; i < couponList.rows.length; i++) {
          if (couponList.rows[i]._deleted
            || couponList.rows[i].id == coupon._id
            || couponList.rows[i].doc.valid_to < validityStart) {
            couponList.rows.splice(i, 1);
            i--;
            continue;
          }
        }
        deferred.resolve(couponList.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    isFavorite: function(couponId) {
      var deferred = $q.defer();
      UserdataDb.get().get('favourite-coupon-' + UserConfig.getId() + '-' + couponId).then(function() {
        deferred.resolve(true);
      }).catch(function(error) {
        if (error.status == 404) {
          deferred.resolve(false);
        }
        else {
          deferred.reject(error);
        }
      });
      return deferred.promise;
    },
    getFavorites: function(validityStart) {
      var deferred = $q.defer();

      var errorHandler = function(error) {
        deferred.reject(error);
      };

      UserdataDb.get().query('favourites/coupons', {reduce: false}).then(function(results) {
        var ids = [];
        for (var c in results.rows) {
          if (!isNaN(parseInt(c))) {
            ids.push(results.rows[c].key);
          }
        }

        UserdataDb.get().query('redeemed/coupons', {reduce: false}).then(function(results) {
          for (var c in results.rows) {
            if (!isNaN(parseInt(c))) {
              var i = ids.indexOf(results.rows[c].key);
              if (i >= 0) {
                ids.splice(i, 1);
              }
            }
          }

          GeneralDb.get().allDocs({
            keys: ids,
            include_docs: true,
            attachments: true
          }).then(function(result) {
            var clist = [];
            for (var c in result.rows) {
              if (!isNaN(parseInt(c)) && !result.rows[c].error) {
                //console.log(JSON.stringify(result.rows[c]));
                var validTo = new Date(result.rows[c].doc.valid_to);
                if (validTo.getTime() >= new Date(validityStart).getTime()) {
                  clist.push(result.rows[c].doc);
                }
              }
            }
            deferred.resolve(clist);
          }).catch(errorHandler);
        }).catch(errorHandler);
      }).catch(errorHandler);
      return deferred.promise;
    },
    getAllRedeems: function() {
      return functions.getRowsArrayPromise(UserdataDb.get().query('redeemed/coupons', {reduce: false}));
    },
    redeem: function(id) {
      var deferred = $q.defer();
      var doc = {
        _id: 'redeemed-coupon-' + UserConfig.getId() + '-' + id,
        type: 'redeemed-coupon',
        user_id: UserConfig.getId(),
        coupon_id: id,
        timestamp: new Date().getTime()
      };
      UserdataDb.get().put(doc).then(function(response) {
        $log.info('redeem error: ' + JSON.stringify(response));
        if (response.ok) {
          deferred.resolve();
        }
        else {
          deferred.reject('could not insert favourite into the db');
        }
      }).catch(function(err) {
        deferred.reject(err);
      });
      return deferred.promise;
    },
    like: function(id) {
      var deferred = $q.defer();
      UserdataDb.get().get('favourite-coupon-' + UserConfig.getId() + '-' + id).then(function(coupon) {
        UserdataDb.get().remove(coupon).then(function() {
          deferred.resolve(false);
        }).catch(function(err) {
          deferred.reject(err);
        });
      }).catch(function() {
        var doc = {
          _id: 'favourite-coupon-' + UserConfig.getId() + '-' + id,
          type: 'favourite-coupon',
          user_id: UserConfig.getId(),
          coupon_id: id,
          timestamp: new Date().getTime()
        };
        UserdataDb.get().put(doc).then(function(response) {
          $log.info('like error: ' + JSON.stringify(response));
          if (response.ok) {
            deferred.resolve(true);
          }
          else {
            deferred.reject('could not insert favourite into the db');
          }
        }).catch(function(err) {
          deferred.reject(err);
        });
      });
      return deferred.promise;
    },
  };
  return functions;
});

persistency.factory('OfferDAO', function($q, $log, GeneralDb) {
  var functions = {
    getRowsArrayPromise: function(promise) {
      var deferred = $q.defer();
      promise.then(function(result) {
        deferred.resolve(result.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    getOffersByValidity: function(validityStart) {
      return functions.getRowsArrayPromise(GeneralDb.get().query('offers/byValidity', {
        startkey: validityStart,
        reduce: false,
        include_docs: true,
        attachments: true
      }));
    },
    getOffersByCategory: function(category, validityStart, exclude_id) {
      var deferred = $q.defer();
      GeneralDb.get().query('offers/byCategory', {
        reduce: false,
        keys: category,
        include_docs: true,
        attachments: true
      }).then(function(offerList) {
        for (var i = 0; i < offerList.rows.length; i++) {
          if (offerList.rows[i]._deleted
            || offerList.rows[i].doc._id == exclude_id
            || offerList.rows[i].doc.valid_to < validityStart) {
            offerList.rows.splice(i, 1);
            i--;
            continue;
          }
        }
        deferred.resolve(offerList.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    getOffersByShop: function(shopId, validityStart) {
      var deferred = $q.defer();
      GeneralDb.get().query('offers/byShop', {
        reduce: false,
        key: shopId,
        include_docs: true
      }).then(function(offerList) {
        for (var i = 0; i < offerList.rows.length; i++) {
          if (offerList.rows[i]._deleted
            || offerList.rows[i].id == offer._id
            || offerList.rows[i].doc.valid_to < validityStart) {
            offerList.rows.splice(i, 1);
            i--;
            continue;
          }
        }
        deferred.resolve(offerList.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    getSimilarOfferCount: function(offer, validityStart) {
      var deferred = $q.defer();

      GeneralDb.get().query('offers/byCategory', {
        reduce: false,
        keys: offer.category,
        include_docs: true
      }).then(function(offerList) {
        for (var i = 0; i < offerList.rows.length; i++) {
          if (offerList.rows[i]._deleted
            || offerList.rows[i].doc._id == offer._id
            || offerList.rows[i].doc.valid_to < validityStart) {
            offerList.rows.splice(i, 1);
            i--;
            continue;
          }
        }
        deferred.resolve(offerList.rows.length);

      }).catch(function(error) {
        deferred.resolve(0);
      });
      return deferred.promise;
    }
  };
  return functions;
});


persistency.factory('ShopDAO', function($q, $log, GeneralDb, UserdataDb) {
  var functions = {
    getRowsArrayPromise: function(promise) {
      var deferred = $q.defer();
      promise.then(function(result) {
        deferred.resolve(result.rows);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    count: function() {
      var deferred = $q.defer();
      GeneralDb.get().query('shops/all')
        .then(function(result) {
          deferred.resolve(result.rows[0].value);
        })
        .catch(function(error) {
          $log.error('Shop count error: ' + JSON.stringify(error));
          deferred.resolve(0);
        });
      return deferred.promise;
    },
    getAll: function() {
      return functions.getRowsArrayPromise(GeneralDb.get().query('shops/all', {
        reduce: false,
        include_docs: true,
        attachments: true
      }));
    },
    getFavorites: function() {
      var deferred = $q.defer();

      var errorHandler = function(error) {
        deferred.reject(error);
      };

      UserdataDb.get().query('favourites/shops', {reduce: false}).then(function(results) {
        var keys = [];
        for (var fs in results.rows) {
          if (!isNaN(parseInt(fs)) && !results.rows[fs]._deleted) {
            keys.push(results.rows[fs].key);
          }
        }
        GeneralDb.get().allDocs({
          include_docs: true,
          attachments: true,
          keys: keys
        }).then(function(result) {
          var clist = [];
          for (var c in result.rows) {
            if (!isNaN(parseInt(c))) {
              clist.push(result.rows[c].doc);
            }
          }
          deferred.resolve(clist);
        }).catch(errorHandler);
      }).catch(errorHandler);
      return deferred.promise;
    },
    getShopsByCategory: function(category) {
      return functions.getRowsArrayPromise(GeneralDb.get().query('shops/byCategory', {
        reduce: false,
        include_docs: true,
        attachments: true,
        keys: category
      }));
    },
    getSimilarShopCount: function(categories) {
      var deferred = $q.defer();
      GeneralDb.get().query('shops/byCategory', {
        group: true,
        keys: categories,
      }).then(function(similarShopStat) {
        var count = 0;
        for (var i = 0; i < similarShopStat.rows.length; i++) {
          count += similarShopStat.rows[i].value;
        }
        count -= categories.length;
        if (count < 0) {
          count = 0;
        }
        deferred.resolve(count);
      }).catch(function(error) {
        $log.error('getSimilarShopCount error: ' + JSON.stringify(error));
        deferred.resolve(0);
      });
      return deferred.promise;
    },
    getFavoriteShop: function(userId, shopId) {
      return UserdataDb.get().get('favourite-shop-' + userId + '-' + shopId, {conflicts: true, revs_info: true});
    },
    isShopFavorite: function(userId, shopId) {
      var deferred = $q.defer();
      functions.getFavoriteShop(userId, shopId).then(function(doc) {
        deferred.resolve(true);
      }).catch(function(error) {
        if (error.status == 404) {
          deferred.resolve(false);
        }
        else {
          deferred.reject(error);
        }
      });
      return deferred.promise;
    },
    removeFavoriteShop: function(favShop) {
      return UserdataDb.get().remove(favShop);
    },
    insertFavoriteShop: function(userId, shopId) {
      var doc = {
        _id: 'favourite-shop-' + userId + '-' + shopId,
        type: 'favourite-shop',
        user_id: userId,
        shop_id: shopId,
        timestamp: new Date().getTime()
      };
      return UserdataDb.get().put(doc);
    }
  };
  return functions;
});


persistency.factory('FavoriteDAO', function($q, $log, UserConfig, GeneralDb, UserdataDb) {
  var functions = {
    getFavoriteProducts: function() {
      var deferred = $q.defer();

      GeneralDb.get().query('products/ordered', {
        reduce: false,
        include_docs: true,
        attachments: true
      }).then(function(results) {
        var plist = [];
        var pids = [];
        for (var c in results.rows) {
          if (!isNaN(parseInt(c))) {
            var doc = results.rows[c].doc;
            doc.favourite = false;
            plist.push(doc);
            pids.push('favourite-product-' + UserConfig.getId() + '-' + results.rows[c].id);
          }
        }

        UserdataDb.get().allDocs({
          keys: pids,
          include_docs: true,
          attachments: true
        }).then(function(result) {
          for (var c in result.rows) {
            if (!isNaN(parseInt(c))) {
              for (var p in plist) {
                //console.log(result.rows[c]);
                if (!isNaN(parseInt(p)) && result.rows[c].doc != null && result.rows[c].doc.product_id == plist[p]._id) {
                  plist[p].favourite = true;
                  break;
                }
              }
            }
          }
          deferred.resolve(plist);
        }).catch(function(err) {
          deferred.resolve(plist);
        });
      }).catch(function(err) {
        deferred.reject(err);
      });

      return deferred.promise;
    },
    getMessagesByValidity: function(startValidity) {
      var deferred = $q.defer();

      GeneralDb.get().query('messages/byValidity', {
        reduce: false,
        startkey: startValidity
      }).then(function(results) {
        var msgList = [];
        for (var i = 0; i < results.rows.length; i++) {
          if (!results.rows[i]._deleted
            && results.rows[i].value[0] < now) {
            results.rows[i].read = UserConfig.isReadMessage(results.rows[i].id);
            msgList.push(results.rows[i]);
          }
        }
        deferred.resolve(msgList);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },
    getFavoriteProduct: function(userId, productId) {
      return UserdataDb.get().get('favourite-product-' + userId + '-' + productId, {conflicts: true, revs_info: true});
    },
    removeFavoriteProduct: function(product) {
      return UserdataDb.get().remove(product);
    },
    insertFavoriteProduct: function(userId, productId) {
      var doc = {
        _id: 'favourite-product-' + userId + '-' + productId,
        type: 'favourite-product',
        user_id: userId,
        product_id: productId,
        timestamp: new Date().getTime()
      };
      return UserdataDb.get().put(doc);
    }
  };
  return functions;
});

/**
 * Stores redeemed coupons statistics, the coupon amounts will be reduced by these numbers
 */
persistency.factory('RedeemStatDAO', function($q, $log, UserdataRemoteDb, GeneralDb) {
  var doc_id = '_local/redeemed-coupons';
  var functions = {

    getDoc: function() {
      var deferred = $q.defer();
      GeneralDb.get().get(doc_id)
        .then(function(doc) {
          deferred.resolve(doc);
        })
        .catch(function(error) {
          deferred.reject(error);
        });
      return deferred.promise;
    },

    putDoc: function(doc) {
      var deferred = $q.defer();
      functions.getDoc()
        .then(function(previous_doc) {
          doc._rev = previous_doc._rev;
          GeneralDb.get().put(doc)
            .then(function() {
              $log.debug('Coupon stat updated');
              deferred.resolve();
            })
            .catch(function(error) {
              $log.error('Coupon stat could not be updated: ' + JSON.stringify(error));
              deferred.reject(error);
            });
        })
        .catch(function(error) {
          GeneralDb.get().put(doc)
            .then(function() {
              $log.debug('Coupon stat updated');
              deferred.resolve();
            })
            .catch(function(error) {
              $log.error('Coupon stat could not be updated: ' + JSON.stringify(error));
              deferred.reject(error);
            });
        });
      return deferred.promise;
    },

    update: function() {
      var deferred = $q.defer();

      var remotedb = UserdataRemoteDb.get();

      remotedb.query('redeemed/coupons', {
        reduce: true,
        group: true
      })
        .then(function(redeemed_coupon_stat) {

          var id = '_local/redeemed-coupons';
          var doc = {_id: id};
          for (var i = 0; i < redeemed_coupon_stat.rows.length; i++) {
            doc[redeemed_coupon_stat.rows[i].key] = redeemed_coupon_stat.rows[i].value;
          }

          functions.putDoc(doc).finally(function() {
            deferred.resolve();
          });

        })
        .catch(function(error) {
          $log.error('Coupon stat could not be retrieved: ' + JSON.stringify(error));
          deferred.resolve();
        });

      return deferred.promise;
    },

    getRedeems: function(coupon_id) {
      var deferred = $q.defer();

      functions.getDoc()
        .then(function(doc) {
          if (doc[coupon_id]) {
            deferred.resolve(doc[coupon_id]);
          }
          else {
            deferred.resolve(0);
          }
        })
        .catch(function(error) {
          $log.debug(error);
          deferred.resolve(0);
        });

      return deferred.promise;
    },

  };
  return functions;
});
