var services = angular.module('premieroutlet.services', ['premieroutlet.persistency']);

services.factory('Today', function($filter) {
  return $filter('date')(Date.now(), 'yyyy-MM-dd');
});

services.factory('Yesterday', function($filter) {
  return $filter('date')(Date.now() - (1000 * 60 * 60 * 24), 'yyyy-MM-dd');
});


services.factory('ListBuilderService', function($q, $rootScope, $log, Today, UserConfig, BasicDAO, CouponDAO, OfferDAO, ShopDAO, FavoriteDAO) {
  var functions = {

    addImagesFromShop: function(itemList, itemAttachmentName, shopAttachmentName) {
      var deferred = $q.defer();

      if (!(itemList instanceof Array)) {
        itemList = [itemList];
      }

      var noimageIds = [];
      for (var c in itemList) {
        if (!isNaN(parseInt(c))) {
          var item = itemList[c];
          if (!item._attachments || !item._attachments[itemAttachmentName]) {
            if (!noimageIds.includes(item.shop_id)) {
              noimageIds.push(item.shop_id);
            }
          }
        }
      }

      if (noimageIds.length > 0) {
        BasicDAO.getDocsById(noimageIds).then(function(shops) {
          for (var s in shops) {
            if (!isNaN(parseInt(s))) {
              var sid = shops[s].id;
              for (var c in itemList) {
                if (!isNaN(parseInt(c))) {
                  if (itemList[c].shop_id == sid && (!itemList[c]._attachments || !itemList[c]._attachments[itemAttachmentName]) && shops[s].doc && shops[s].doc._attachments) {
                    if (!itemList[c]._attachments) {
                      itemList[c]._attachments = {};
                    }
                    itemList[c]._attachments[itemAttachmentName] = shops[s].doc._attachments[shopAttachmentName];
                  }
                }
              }
            }
          }
          deferred.resolve(itemList);
        }).catch(function(error) {
          console.log('Shops cannot be retrieved for coupon list images: ' + JSON.stringify(error));
          deferred.reject(error);
        });
      }
      else {
        deferred.resolve(itemList);
      }
      return deferred.promise;
    },

    buildCouponList: function(result) {
      var deferred = $q.defer();
      var clist = [];
      for (var c in result) {
        if (!isNaN(parseInt(c))) {
          clist.push(result[c].doc);
        }
      }

      CouponDAO.getAllRedeems().then(function(results) {
        var redeemedIds = [];
        for (var c in results) {
          if (!isNaN(parseInt(c))) {
            redeemedIds.push(results[c].key);
          }
        }

        var filteredList = [];
        for (var c in clist) {
          if (!isNaN(parseInt(c)) && !redeemedIds.includes(clist[c]._id)) {
            filteredList.push(clist[c]);
          }
        }

        functions.addImagesFromShop(filteredList, 'listimage.png', 'logo.png').then(function(couponListWithImages) {
          deferred.resolve(couponListWithImages);
        }).catch(function(error) {
          $log.warn('Could not add images to coupons from shops: ' + error);
          deferred.resolve(filteredList);
        });
      }).catch(function(err) {
        if (err.status == 404) {
          $log.debug('No redeemed coupons found');
          functions.addImagesFromShop(clist, 'listimage.png', 'logo.png').then(function(couponListWithImages) {
            deferred.resolve(couponListWithImages);
          }).catch(function(error) {
            $log.warn('Could not add images to coupons from shops: ' + error);
            deferred.resolve(clist);
          });
        }
        else {
          deferred.reject(err);
        }
      });
      return deferred.promise;
    },

    buildValidCouponsList: function() {
      $log.debug('Building coupon list...')
      var deferred = $q.defer();

      CouponDAO.getCouponsByValidity(Today).then(function(validCouponList) {
        functions.buildCouponList(validCouponList).then(function(coupons) {
          deferred.resolve(coupons);
        }).catch(function(error) {
          $log.error('buildValidCouponsList buildCouponList error: ' + JSON.stringify(error));
          deferred.reject(error);
        });
      }).catch(function(error) {
        $log.error('buildValidCouponsList getCouponsByValidity error: ' + JSON.stringify(error));
        deferred.reject(error);
      });

      return deferred.promise;
    },

    buildOfferList: function(result) {
      var deferred = $q.defer();
      var clist = [];
      var priorityOffers = [];
      for (var c in result) {
        if (!isNaN(parseInt(c))) {
          clist.push(result[c].doc);
          if (result[c].doc.shop_id == 'shop-0') {
            priorityOffers.push(c);
          }
        }
      }
      for (var pi in priorityOffers) {
        clist.moveToFirst(priorityOffers[pi]);
      }

      functions.addImagesFromShop(clist, 'listimage.png', 'logo.png').then(function(offerListWithImages) {
        deferred.resolve(offerListWithImages);
      }).catch(function(error) {
        $log.warn('Could not add images to offers from shops: ' + error);
        deferred.resolve(clist);
      });
      return deferred.promise;
    },

    buildValidOffersList: function() {
      var deferred = $q.defer();

      OfferDAO.getOffersByValidity(Today).then(function(validOfferList) {
        functions.buildOfferList(validOfferList).then(function(offers) {
          deferred.resolve(offers);
        }).catch(function(error) {
          $log.error('buildOfferList error: ' + JSON.stringify(error));
          deferred.reject(error);
        });
      }).catch(function(error) {
        $log.error('getOffersByValidity error: ' + JSON.stringify(error));
        deferred.reject(error);
      });
      return deferred.promise;
    },

    buildShopList: function(result) {
      var clist = [];
      for (var c in result) {
        if (!isNaN(parseInt(c))) {
          clist.push(result[c].doc);
        }
      }
      return clist;
    },

    buildAllShops: function() {
      var deferred = $q.defer();

      ShopDAO.getAll().then(function(allShops) {
        var clist = functions.buildShopList(allShops);
        deferred.resolve(clist);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },

    buildShopListByFavourite: function(filter) {
      var deferred = $q.defer();
      var promise = (filter) ? (ShopDAO.getFavorites()) : (functions.buildAllShops());
      promise.then(function(result) {
        deferred.resolve(result);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },

    buildShopListByCategory: function(category) {
      var deferred = $q.defer();
      ShopDAO.getShopsByCategory([category]).then(function(result) {
        deferred.resolve(functions.buildShopList(result));
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },

    buildFavouriteCouponsList: function() {
      return CouponDAO.getFavorites(Today);
    },

    buildFavouriteShopsList: function() {
      return ShopDAO.getFavorites();
    },

    buildFavouriteProductsList: function() {
      return FavoriteDAO.getFavoriteProducts();
    },

    buildMessagesList: function() {
      var now = Math.floor(new Date().getTime() / 1000);
      return FavoriteDAO.getMessagesByValidity(now);
    },

  };
  return functions;
});


services.factory('DataLoaderService', function($q, $http, $log, SYNC_INTERVAL, Today, Yesterday,
                                               UserConfig, GeneralRemoteDb, UserdataRemoteDb, RedeemStatDAO) {
  var functions = {

    syncEverything: function(forceSync) {
      if (forceSync === undefined) {
        forceSync = false;
      }
      var deferred = $q.defer();

      /*
       0. No data sync within the time interval
       1. replicate general data from remote to local db
       2. sync user data
       3. load redeemed coupon statistics
       */

      if (!forceSync && (new Date().getTime() - UserConfig.getLastSyncTS()) < SYNC_INTERVAL) {
        $log.debug('Sync interval is not yet over');
        deferred.resolve(0);
      }
      else {
        try {
          $log.debug('Sync everything started');

          GeneralRemoteDb.sync()
            .then(function(changeCount) {
              UserdataRemoteDb.sync()
                .then(function() {
                  deferred.resolve(changeCount);
                  RedeemStatDAO.update()
                    .then(function() {
                      $log.debug('Redeemed coupon stat updated');
                    })
                    .catch(function(error) {
                      $log.error('Redeemed coupon stat could not be retrieved: ' + JSON.stringify(error));
                    });
                })
                .catch(function(error) {
                  $log.error('User data sync failed: ' + JSON.stringify(error));
                  deferred.reject(error);
                });
              deferred.resolve(changeCount);
            })
            .catch(function(error) {
              $log.error('General db replication failed: ' + JSON.stringify(error));
              deferred.reject(error);
            });

        }
        catch(ex) {
          deferred.reject(ex);
        }
      }
      return deferred.promise;
    },

  };
  return functions;
});


services.factory('CouponDetailService', function($q, $log, $http, Today, UserConfig, ListBuilderService, BasicDAO, CouponDAO, RedeemStatDAO) {
  var functions = {

    getCoupon: function(id) {
      var deferred = $q.defer();
      BasicDAO.getDocById(id).then(function(coupon) {

        ListBuilderService.addImagesFromShop(coupon, 'listimage.png', 'logo.png').then(function(couponList) {
          coupon = couponList[0];
          coupon.favourite = false;
          coupon.similar_count = 0;

          var handlerFavouriteCoupon = function(favourite) {
            coupon.favourite = favourite;
          };

          var handlerSimilarCouponCount = function(couponList) {
            for (var i = 0; i < couponList.length; i++) {
              if (couponList[i]._deleted
                || couponList[i].doc._id == id
                || couponList[i].doc.valid_to < Today) {
                couponList.splice(i, 1);
                i--;
                continue;
              }
            }
            coupon.similar_count = couponList.length;
          };

          var promises = [
            CouponDAO.isFavorite(coupon._id),
            CouponDAO.getCouponsByCategory(coupon.category)
          ];

          var handlers = [
            handlerFavouriteCoupon,
            handlerSimilarCouponCount
          ];

          $q.all(promises).then(function(results) {
            for (var i = 0; i < results.length; i++) {
              handlers[i](results[i]);
            }
            deferred.resolve(coupon);
          }).catch(function(errors) {
            deferred.resolve(errors);
          });

        }).catch(function(error) {
          $log.error('Could not load image from shop to coupon: ' + JSON.stringify(error));
        });

      }).catch(function(err) {
        deferred.reject(err);
      });
      return deferred.promise;
    },

    getSimilarCoupons: function(id) {
      var deferred = $q.defer();

      var errorCatcher = function(error) {
        $log.error('getSimilarCoupons error: ' + JSON.stringify(error));
        deferred.reject(error);
      };

      BasicDAO.getDocById(id).then(function(coupon) {
        CouponDAO.getCouponsByCategory(coupon.category).then(function(couponList) {
          for (var i = 0; i < couponList.length; i++) {
            if (couponList[i]._deleted
              || couponList[i].doc._id == id
              || couponList[i].doc.valid_to < Today) {
              couponList.splice(i, 1);
              i--;
              continue;
            }
          }
          ListBuilderService.buildCouponList(couponList).then(function(coupons) {
            deferred.resolve(coupons);
          }).catch(errorCatcher);
        }).catch(errorCatcher);
      }).catch(errorCatcher);
      return deferred.promise;
    },

    redeemCoupon: function(id) {
      return CouponDAO.redeem(id);
    },

    likeCoupon: function(id) {
      return CouponDAO.like(id);
    },

    getRedeemedCouponAmount: function(id) {
      return RedeemStatDAO.getRedeems(id);
    },

    getMoreCoupons: function(coupon) {
      return CouponDAO.getCouponsByShop(coupon.shop_id, Today);
    },

  };
  return functions;
});



services.factory('ShopDetailService', function($q, $log, $http, UserConfig, ListBuilderService, BasicDAO, ShopDAO) {
  var functions = {

    getCategories: function() {
      var deferred = $q.defer();
      ShopDAO.getAll().then(function(shopList) {
        var cats = {};
        for (var sh in shopList) {
          if (!isNaN(sh) && shopList[sh].doc && shopList[sh].doc.category) {
            for (var ca in shopList[sh].doc.category) {
              if (!isNaN(ca)) {
                var cc = shopList[sh].doc.category[ca];
                //console.log(cc);
                if (cats[cc]) {
                  cats[cc] += 1;
                }
                else {
                  cats[cc] = 1;
                }
              }
            }
          }
        }

        var categories = [];
        for (var p in cats) {
          if (cats.hasOwnProperty(p)) {
            categories.push(p);
          }
        }
        deferred.resolve(categories);

      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },

    getShop: function(id) {
      var deferred = $q.defer();
      BasicDAO.getDocById(id).then(function(shop) {
        ShopDAO.getSimilarShopCount(shop.category).then(function(count) {
          shop.similar_count = count;
        }).catch(function(error) {
          $log.error('getSimilarShopCount error: ' + JSON.stringify(error));
          shop.similar_count = 0;
        });
        ShopDAO.isShopFavorite(UserConfig.getId(), shop._id).then(function(favorite) {
          shop.favourite = favorite;
          deferred.resolve(shop);
        }).catch(function(error) {
          $log.error('isShopFavorite error: ' + JSON.stringify(error));
          shop.favourite = false;
          deferred.resolve(shop);
        });
      }).catch(function(error) {
        $log.error('getShop getDocById error: ' + JSON.stringify(error));
        deferred.reject(error);
      });

      return deferred.promise;
    },

    getSimilarShops: function(id) {
      var deferred = $q.defer();

      var errorCatcher = function(error) {
        $log.error('getSimilarShops error: ' + JSON.stringify(error));
        deferred.reject(error);
      };

      BasicDAO.getDocById(id).then(function(shop) {
        ShopDAO.getShopsByCategory(shop.category).then(function(shopList) {
          for (var i = 0; i < shopList.length; i++) {
            if (shopList[i].doc._id == id) {
              shopList.splice(i, 1);
              break;
            }
          }
          deferred.resolve(ListBuilderService.buildShopList(shopList));
        }).catch(errorCatcher);
      }).catch(errorCatcher);
      return deferred.promise;
    },

    likeShop: function(id) {
      var deferred = $q.defer();
      ShopDAO.getFavoriteShop(UserConfig.getId(), id).then(function(shop) {
        ShopDAO.removeFavoriteShop(shop).then(function() {
          deferred.resolve(false);
        }).catch(function(err) {
          deferred.reject(err);
        });
      }).catch(function(error) {
        if (error.status == 404) {
          ShopDAO.insertFavoriteShop(UserConfig.getId(), id).then(function(response) {
            console.log(response);
            if (response.ok) {
              deferred.resolve(true);
            }
            else {
              deferred.reject('could not insert favourite into the db');
            }
          }).catch(function(err) {
            deferred.reject(err);
          });
        }
        else {
          deferred.reject('database error');
        }
      });
      return deferred.promise;
    },

  };
  return functions;
});


services.factory('OfferDetailService', function($q, $log, Today, ListBuilderService, BasicDAO, OfferDAO) {
  var functions = {

    getOffer: function(id) {
      var deferred = $q.defer();

      BasicDAO.getDocById(id).then(function(offer) {
        ListBuilderService.addImagesFromShop(offer, 'listimage.png', 'logo.png').then(function(offerList) {
          offer = offerList[0];
          offer.similar_count = 0;

          OfferDAO.getSimilarOfferCount(offer, Today).then(function(count) {
            offer.similar_count = count;
            deferred.resolve(offer);
          }).catch(function(error) {
            $log.error('getSimilarOfferCount error: ' + JSON.stringify(error));
            deferred.resolve(offer);
          });
        }).catch(function(error) {
          $log.error('Could not load image from shop to coupon: ' + JSON.stringify(error));
          deferred.resolve(offer);
        });
      }).catch(function(err) {
        deferred.reject(err);
      });
      return deferred.promise;
    },

    getSimilarOffers: function(id) {
      var deferred = $q.defer();

      var errorCatcher = function(error) {
        $log.error('getSimilarOffers error: ' + JSON.stringify(error));
        deferred.reject(error);
      };

      BasicDAO.getDocById(id).then(function(offer) {
        OfferDAO.getOffersByCategory(offer.category, Today, offer._id).then(function(offerList) {
          ListBuilderService.buildOfferList(offerList).then(function(offers) {
            deferred.resolve(offers);
          }).catch(errorCatcher);
        }).catch(errorCatcher);
      }).catch(errorCatcher);
      return deferred.promise;
    },

    getMoreOffers: function(offer) {
      return OfferDAO.getOffersByShop(offer.shop_id, Today);
    },

  };
  return functions;
});


services.factory('FavouritesService', function($q, $log, $http, Config, UserConfig, UserdataRemoteDb, DataLoaderService, ngFB, ngGP, BasicDAO, FavoriteDAO) {
  var functions = {

    fbLogin: function() {
      var deferred = $q.defer();
      ngFB.login({scope: 'email'}).then(function (response) {
        if (response.status === 'connected') {
          $log.info('Facebook login success.');
          $log.debug(response);
          deferred.resolve(response);
        }
        else {
          //console.log('Facebook login failed');
          deferred.reject('Sikertelen bejelentkezés');
        }
      });
      return deferred.promise;
    },

    fbSaveLoginData: function(accessToken) {
      var deferred = $q.defer();
      $log.debug('Loading user data from facebook');
      var xhr = new XMLHttpRequest();
      xhr.open('GET', 'https://graph.facebook.com/me?access_token=' + accessToken, true);
      xhr.onload = function() {
        //console.log(xhr);
        if (xhr.status == 200) {
          var jsonUser = xhr.responseText;
          var user = JSON.parse(jsonUser);
          $log.info('User:' + JSON.stringify(user));

          var config = Config.get();
          var loginUrl = config.loginUrl + '/facebook/';
          console.log('Login url: ' + loginUrl);
          xhr.open('POST', loginUrl, true);
          xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
          xhr.onload = function() {
            $log.info('logged in' + JSON.stringify(xhr));
            if (xhr.status == 200) {
              var user = JSON.parse(xhr.responseText);
              $log.info('user error: ' + JSON.stringify(user));
              UserConfig.saveId(user._id);
              UserConfig.saveStringData(xhr.responseText);
              UserdataRemoteDb.sync().then(function() {
                console.log('User db url retrieved and synced');
                deferred.resolve();
              }).catch(function(err) {
                console.log('User data could not be synced: ' + JSON.stringify(err));
                deferred.reject(err);
              });
            }
            else {
              deferred.reject('Bejelentkezési adatok tárolása sikertelen.');
            }
          };
          //console.log('user=' + encodeURIComponent(jsonUser));
          xhr.send(jsonUser);

        }
        else {
          console.log('Unexpected status: ' + xhr.status);
          deferred.reject('Adatok betöltése sikertelen.');
        }
      }
      xhr.send(null);
      return deferred.promise;
    },

    gpLogin: function() {
      var deferred = $q.defer();
      ngGP.login().then(function(result) {
        console.log('Login successful: ' + JSON.stringify(result));
        deferred.resolve(result);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },

    gpSaveLoginData: function(accessToken) {
      var deferred = $q.defer();
      var userinfoUrl = 'https://www.googleapis.com/oauth2/v1/userinfo?alt=json&access_token=' + accessToken;
      console.log('Loading user data from ' + userinfoUrl);
      var xhr = new XMLHttpRequest();
      xhr.open('GET', userinfoUrl, true);
      xhr.onload = function() {
        //console.log(xhr);
        if (xhr.status == 200) {
          var jsonUser = xhr.responseText;
          var user = JSON.parse(jsonUser);
          $log.info('User:' + JSON.stringify(user));

          var config = Config.get();
          var loginUrl = config.loginUrl + '/googleplus';
          console.log('Login url: ' + loginUrl);
          xhr.open('POST', loginUrl, true);
          xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
          xhr.onload = function() {
            console.log('logged in');
            //console.log(xhr);
            if (xhr.status == 200) {
              var user = JSON.parse(xhr.responseText);
              $log('User error: ' + JSON.stringify(user));
              UserConfig.saveId(user._id);
              UserConfig.saveStringData(xhr.responseText);
              UserdataRemoteDb.sync().then(function() {
                console.log('User db url retrieved and synced');
                deferred.resolve();
              }).catch(function(err) {
                console.log('User data could not be synced: ' + JSON.stringify(err));
                deferred.reject(err);
              });
            }
            else {
              deferred.reject('Bejelentkezési adatok tárolása sikertelen.');
            }
          };
          //console.log('user=' + encodeURIComponent(jsonUser));
          xhr.send(jsonUser);
        }
        else {
          console.log('Unexpected status: ' + xhr.status);
          deferred.reject('Adatok betöltése sikertelen.');
        }
      }
      xhr.send(null);
      return deferred.promise;
    },


    ownLogin: function(userdata) {
      var deferred = $q.defer();
      var config = Config.get();
      var loginUrl = config.loginUrl + '/own';
      console.log('Login url: ' + loginUrl);
      var xhr = new XMLHttpRequest();
      xhr.open('POST', loginUrl, true);
      xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
      xhr.onload = function() {
        console.log('logged in');
        //console.log(xhr);
        if (xhr.status == 200) {
          var user = JSON.parse(xhr.responseText);
          $log.info(JSON.stringify(user));
          UserConfig.saveId(user._id);
          UserConfig.saveStringData(xhr.responseText);
          UserdataRemoteDb.sync().then(function() {
            console.log('User db url retrieved and synced');
            deferred.resolve();
          }).catch(function(err) {
            console.log('User data could not be synced: ' + JSON.stringify(err));
            deferred.reject(err);
          });
        }
        else {
          deferred.reject('Bejelentkezési adatok tárolása sikertelen.');
        }
      };
      //console.log('user=' + encodeURIComponent(jsonUser));
      xhr.send(JSON.stringify(userdata));

      return deferred.promise;
    },


    checkUserStatus: function() {
      var deferred = $q.defer();
      var config = Config.get();
      var userStatusUrl = config.userStatusUrl + '/' + UserConfig.getId();
      console.log('User status url: ' + userStatusUrl);
      var xhr = new XMLHttpRequest();
      xhr.open('GET', userStatusUrl, true);
      xhr.onload = function() {
        console.log('user status:' + xhr.responseText);
        if (xhr.status == 200) {
          var status = JSON.parse(xhr.responseText);
          var user = UserConfig.getData();
          if (user.needs_confirm !== !status.active) {
            user.needs_confirm = !status.active;
            UserConfig.saveData(user);
            if (status.active) {
              UserdataRemoteDb.sync().then(function() {
                console.log('User db url retrieved and synced');
                deferred.resolve(true);
              }).catch(function(err) {
                console.log('User data could not be synced: ' + JSON.stringify(err));
                deferred.reject(err);
              });
            }
            else {
              deferred.resolve(status.active);
            }
          }
          else {
            deferred.resolve(status.active);
          }
        }
        else {
          deferred.reject('Aktiválás ellenőrzése sikertelen.');
        }
      };
      xhr.send(null);

      return deferred.promise;
    },


    resendActivationEmail: function() {
      var deferred = $q.defer();
      var config = Config.get();
      var resendUrl = config.resendUrl + '/' + UserConfig.getId();
      console.log('Resend url: ' + resendUrl);
      var xhr = new XMLHttpRequest();
      xhr.open('GET', resendUrl, true);
      xhr.onload = function() {
        console.log('resend status:' + xhr.responseText);
        if (xhr.status == 200) {
          var status = JSON.parse(xhr.responseText);
          deferred.resolve(status);
        }
        else {
          deferred.reject('Email újraküldése sikertelen.');
        }
      };
      xhr.send(null);

      return deferred.promise;
    },


    likeProduct: function(id) {
      var deferred = $q.defer();
      FavoriteDAO.getFavoriteProduct(UserConfig.getId(), id).then(function(product) {
        FavoriteDAO.removeFavoriteProduct(product).then(function() {
          UserdataRemoteDb.sync().catch(function(error) {
            $log.error('removeFavoriteProduct error' + JSON.stringify(error));
          });
          deferred.resolve(false);
        }).catch(function(err) {
          deferred.reject(err);
        });
      }).catch(function() {
        FavoriteDAO.insertFavoriteProduct(UserConfig.getId(), id).then(function(response) {
          //console.log(response);
          UserdataRemoteDb.sync().catch(function(error) {
            $log.error('UserdataRemoteDb sync error: ' + JSON.stringify(error));
          });
          if (response.ok) {
            deferred.resolve(true);
          }
          else {
            deferred.reject('could not insert favourite into the db');
          }
        }).catch(function(err) {
          deferred.reject(err);
        });
      });
      return deferred.promise;
    },

    getProduct: function(id) {
      return BasicDAO.getDocById(id);
    },

    /*
    removeUserData: function() {
      var deferred = $q.defer();
      UserdataDb.get().destroy().then(function(response) {
        console.log('User data db deleted');
        UserConfig.cleanUp();
        deferred.resolve();
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    }
    */

  };
  return functions;
});

services.factory('ReminderDetailService', function($q, $http, UserConfig, ListBuilderService, BasicDAO) {
  var functions = {

    getMessage: function(id) {
      return BasicDAO.getDocById(id);
    },

    markMessageAsRead: function(id) {
      var deferred = $q.defer();
      UserConfig.addReadMessageId(id);
      ListBuilderService.buildMessagesList().then(function(msgList) {
        deferred.resolve(msgList);
      }).catch(function(error) {
        deferred.reject(error);
      });
      return deferred.promise;
    },

  };
  return functions;
});

services.factory('NotificationService', function($q, $http, $log, GOOGLE_PROJECT_NUMBER, Config, UserConfig) {

  var displayMessage = null;
  var push = null;

  var functions = {

    init: function(displayMsg) {
      if (window.PushNotification) {
        if (!push) {
          $log.debug('NotificationService started');
          displayMessage = displayMsg;
          push = PushNotification.init({
            'android': {
              'senderID': GOOGLE_PROJECT_NUMBER,
              'icon': 'icon'
            },
            'ios': {
              'alert': 'true',
              'badge': 'true',
              'sound': 'true'
            }
          });
          push.on('registration', functions.onRegistration);
          push.on('notification', functions.onNotification);
          push.on('error', functions.onError);
        }
        else {
          $log.debug('Push notification has already been initialized.');
        }
      }
      else {
        $log.warn('Push Notification is not available');
      }
    },

    onRegistration: function(data) {
      if (!UserConfig.getId()) {
        return;
      }
      $log.debug('Notification registration: ' + JSON.stringify(data));
      if (data.registrationId && data.registrationId != UserConfig.getNotificationRegistrationId()) {
        $log.debug('Saving notification registration id to remote server');
        var config = Config.get();
        var url = config.notificationRegistrationUrl;
        var params = {
          'userid': UserConfig.getId(),
          'registrationid': data.registrationId,
          'platform': ionic.Platform.platform()
        };
        $log.debug('Params: ' + JSON.stringify(params));
        $http.post(url, params).then(function(response) {
          if (response.status == 200) {
            UserConfig.saveNotificationRegistrationId(data.registrationid);
            $log.debug('Notification registration saved.');
          }
          else {
            $log.debug('Notification registration response: ' + JSON.stringify(response));
          }
        }, function(error) {
          $log.error('Error: ' + JSON.stringify(error));
        });
      }
      else {
        $log.debug('Notification registration id has already been saved.');
      }
    },

    onNotification: function(data) {
      $log.info('Notification received: ' + JSON.stringify(data));
      if (data.additionalData.foreground) {
        displayMessage(data.message);
      }
      if (push) {
        push.finish(function() {
          $log.info('Notification has been processed.');
        });
      }
    },

    onError: function(error) {
      $log.error(JSON.stringify(error));
    },
  };

  return functions;
});

services.factory('ContentService', function($log, BasicDAO) {
  var functions = {
    getInfoContent: function() {
      return BasicDAO.getDocById('info');
    }
  };
  return functions;
});
